@@ -1,9 +1,9 @@
 //
-// "$Id$"
+// "$Id: DiffOpenWindow.h 407 2006-11-13 18:54:02Z mike $"
 //
 // DiffOpenWindow widget definitions.
 //
-// Copyright 2005 by Michael Sweet.
+// Copyright 2005-2006 by Michael Sweet.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License v2 as published
@@ -19,18 +19,18 @@
 #  define _DiffOpenWindow_h_
 #  include <FL/Fl.H>
 #  include <FL/Fl_Double_Window.H>
-#  include <FL/Fl_Return_Button.H>
+#  include <FL/Fl_Button.H>
 #  include <FL/Fl_Preferences.H>
 #  include "DiffChooser.h"
 
 
 class DiffOpenWindow : public Fl_Double_Window
 {
-  DiffChooser		dc1_, dc2_;	// Choosers
-  Fl_Return_Button	compare_;	// Open/Compare button
-  Fl_Button		cancel_;	// Cancel button
+  DiffChooser	dc1_, dc2_;		// Choosers
+  Fl_Button	compare_,		// Open/Compare button
+		cancel_;		// Cancel button
 
-  static void	compare_cb(Fl_Return_Button *b, DiffOpenWindow *dow);
+  static void	compare_cb(Fl_Button *b, DiffOpenWindow *dow);
   static void	cancel_cb(Fl_Button *b, DiffOpenWindow *dow);
   static void	dc_cb(DiffChooser *dc, DiffOpenWindow *dow);
 
@@ -48,5 +48,5 @@ class DiffOpenWindow : public Fl_Double_Window
 #endif // !_DiffOpenWindow_h_
 
 //
-// End of "$Id$".
+// End of "$Id: DiffOpenWindow.h 407 2006-11-13 18:54:02Z mike $".
 //
