@@ -1,9 +1,9 @@
 //
-// "$Id$"
+// "$Id: DiffView.cxx 407 2006-11-13 18:54:02Z mike $"
 //
 // DiffView widget code.
 //
-// Copyright 2005 by Michael Sweet.
+// Copyright 2005-2006 by Michael Sweet.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License v2 as published
@@ -41,8 +41,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <unistd.h>
 
+#include "PtProcess.h"
 
 //
 // Width/height of scrollbars...
@@ -70,15 +70,16 @@ DiffView::DiffView(int        X,	// I - X position
   // Initialize this instance with defaults...
   alloc_        = 0;
   count_        = 0;
-  lines_        = (DiffLine **)0;
-  maxwidth_     = 0;
+  ignoreblanks_ = false;
   linefmt_[0]   = '\0';
   linenum_      = 0;
-  showlinenum_  = true;
-  state_        = IDLE;
-  select_start_ = -1;
+  lines_        = (DiffLine **)0;
+  maxwidth_     = 0;
   select_end_   = -1;
   select_right_ = false;
+  select_start_ = -1;
+  showlinenum_  = true;
+  state_        = IDLE;
   tabwidth_     = 8;
   textcolor_    = FL_BLACK;
   textfont_     = FL_COURIER;
@@ -181,9 +182,13 @@ DiffView::add_line(const char *left,	// I - First line or NULL
   // Update the maximum width of the text...
   if (line->dleft && (leftlen = strlen(line->dleft)) > maxwidth_)
     maxwidth_ = leftlen;
+  else
+    leftlen = 0;
 
   if (line->dright && (rightlen = strlen(line->dright)) > maxwidth_)
     maxwidth_ = rightlen;
+  else
+    rightlen = 0;
 
   if (changed)
   {
@@ -342,9 +347,16 @@ DiffView::draw()
   int w1 = (SCROLLER-4)/3;
   int x1 = xoffset+(SCROLLER-2*w1-1)/2;
   int yy1 = y()+(SCROLLER-w1-1)/2;
-  fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
-  yy1 += h() - 2 * SCROLLER;
-  fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
+  if (Fl::scheme_ && !strcmp(Fl::scheme_, "gtk+")) {
+    fl_polygon(x1, yy1+w1, x1+w1, yy1+w1-1, x1+2*w1, yy1+w1, x1+w1, yy1);
+    yy1 += h()-2*SCROLLER;
+    fl_polygon(x1, yy1, x1+w1, yy1+1, x1+w1, yy1+w1);
+    fl_polygon(x1+w1, yy1+1, x1+2*w1, yy1, x1+w1, yy1+w1);
+  } else {
+    fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
+    yy1 += h()-2*SCROLLER;
+    fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
+  }
 
   // Return early if we have no lines...
   if (!count_)
@@ -474,7 +486,7 @@ DiffView::draw()
   // Draw change markers...
   fl_color(selection_color());
 
-  for (line = lines_, i = 0, ystart = -1; line < last; line ++, i ++)
+  for (line = lines_, i = 0, ystart = -1, yend = -1; line < last; line ++, i ++)
   {
     if ((*line)->changed)
     {
@@ -578,7 +590,7 @@ DiffView::expand_tabs(const char *s)	// I - String to expand
   if (!s)
     return (NULL);
 
-  for (ptr = buf, col = 0; *s && *s != '\n'; s ++)
+  for (ptr = buf, col = 0; *s && *s != '\n' && *s != '\r'; s ++)
     if (*s == '\t')
     {
       do
@@ -668,26 +680,34 @@ DiffView::handle(int event)		// I - Event
 	  {
 	    state_ = UPARROW;
 
-            topline_ -= textsize_;
-	    if (topline_ < 0)
-	      topline_ = 0;
-	    else
-              Fl::add_timeout(0.5f, (void (*)(void *))timeout_cb, (void *)this);
+            if ((count_ * textsize_) > (h() - SCROLLER))
+	    {
+              topline_ -= textsize_;
+	      if (topline_ < 0)
+		topline_ = 0;
+	      else
+        	Fl::add_timeout(0.5f, (void (*)(void *))timeout_cb, (void *)this);
+
+              damage(FL_DAMAGE_SCROLL);
+            }
 
-            damage(FL_DAMAGE_SCROLL);
 	    return (1);
 	  }
           else if (Fl::event_y() >= (y() + h() - 2 * SCROLLER))
 	  {
 	    state_ = DOWNARROW;
 
-            topline_ += textsize_;
-	    if (topline_ > (count_ * textsize_ - h() + SCROLLER))
-	      topline_ = count_ * textsize_ - h() + SCROLLER;
-	    else
-              Fl::add_timeout(0.5f, (void (*)(void *))timeout_cb, (void *)this);
+            if ((count_ * textsize_) > (h() - SCROLLER))
+	    {
+              topline_ += textsize_;
+	      if (topline_ > (count_ * textsize_ - h() + SCROLLER))
+		topline_ = count_ * textsize_ - h() + SCROLLER;
+	      else
+        	Fl::add_timeout(0.5f, (void (*)(void *))timeout_cb, (void *)this);
+
+              damage(FL_DAMAGE_SCROLL);
+	    }
 
-            damage(FL_DAMAGE_SCROLL);
 	    return (1);
 	  }
 	  else if (doscrolling)
@@ -860,11 +880,11 @@ DiffView::load(const char *file1,	// I - First file
                const char *file2)	// I - Second file
 {
   char		command[1024];		// Diff command
-  FILE		*diff,			// File differences
-		*fp;			// First file pointer
+  CPtProcess	pdiff,			// Diff process
+		pfp;			// Original file/directory listing
   const char	*cptr;			// Const pointer into line
   char		*ptr,			// Pointer into line
-		line[1024],		// Line from file/diff
+		line[10240],		// Line from file/diff
 		*dirname,		// Directory name from directory diff
 		*filename,		// Filename from directory diff
 		*left,			// Left file
@@ -878,7 +898,7 @@ DiffView::load(const char *file1,	// I - First file
 		xwidth,			// Column pixel width
 		pos;			// Insert position
 
-
+    
   // Clear the current view...
   clear();
 
@@ -889,12 +909,14 @@ DiffView::load(const char *file1,	// I - First file
   // Do the diff...
   if (file2 && file2[0] != ':')
   {
+    const char *diffopt = ignoreblanks() ? "-bBwE" : "";
+
     if (fl_filename_isdir(file1))
     {
       if (fl_filename_isdir(file2))
       {
-	snprintf(command, sizeof(command), "diff -q \"%s\" \"%s\"",
-	         file1, file2);
+	snprintf(command, sizeof(command), "diff %s -q \"%s\" \"%s\"",
+	         diffopt, file1, file2);
 	directories_ = true;
       }
       else
@@ -904,8 +926,8 @@ DiffView::load(const char *file1,	// I - First file
 	else
           cptr = file2;
 
-	snprintf(command, sizeof(command), "diff -u \"%s/%s\" \"%s\"", file1,
-	         cptr, file2);
+	snprintf(command, sizeof(command), "diff %s -u \"%s/%s\" \"%s\"",
+	         diffopt, file1, cptr, file2);
       }
     }
     else if (fl_filename_isdir(file2))
@@ -915,12 +937,12 @@ DiffView::load(const char *file1,	// I - First file
       else
         cptr = file1;
 
-      snprintf(command, sizeof(command), "diff -u \"%s\" \"%s/%s\"", file1,
-               file2, cptr);
+      snprintf(command, sizeof(command), "diff %s -u \"%s\" \"%s/%s\"",
+               diffopt, file1, file2, cptr);
     }
     else
-      snprintf(command, sizeof(command), "diff -u \"%s\" \"%s\"", file1,
-               file2);
+      snprintf(command, sizeof(command), "diff %s -u \"%s\" \"%s\"", diffopt,
+	       file1, file2);
   }
   else if (fl_filename_isdir(file1))
   {
@@ -934,7 +956,8 @@ DiffView::load(const char *file1,	// I - First file
 	snprintf(command, sizeof(command), "cvs diff -r \"%s\" -u \"%s\"",
 	         file2 + 1, file1);
       else
-	snprintf(command, sizeof(command), "cvs diff -u \"%s\"", file1);
+	snprintf(command, sizeof(command), "cvs diff -u \"%s\"",
+	         file1);
     }
     else
     {
@@ -943,10 +966,10 @@ DiffView::load(const char *file1,	// I - First file
       if (!access(line, 0))
       {
         if (file2)
-          snprintf(command, sizeof(command), "svn status -N -r \"%s\" \"%s\"",
-	           file2 + 1, file1);
+          snprintf(command, sizeof(command), "svn diff -N -r \"%s\" \"%s\"",
+		   file2 + 1, file1);
 	else
-          snprintf(command, sizeof(command), "svn status -N \"%s\"", file1);
+          snprintf(command, sizeof(command), "svn diff -N \"%s\"", file1);
       }
       else
         return (false);
@@ -967,10 +990,10 @@ DiffView::load(const char *file1,	// I - First file
     if (!access(line, 0))
     {
       if (file2)
-	snprintf(command, sizeof(command), "cvs -ql diff -r \"%s\" -u \"%s\"",
+	snprintf(command, sizeof(command), "cvs -q diff -r \"%s\" -u \"%s\"",
 	         file2 + 1, file1);
       else
-	snprintf(command, sizeof(command), "cvs -ql diff -u \"%s\"", file1);
+	snprintf(command, sizeof(command), "cvs -q diff -u \"%s\"", file1);
     }
     else
     {
@@ -978,19 +1001,21 @@ DiffView::load(const char *file1,	// I - First file
 
       if (!access(line, 0))
       {
+        const char *diffopt = ignoreblanks() ?
+	                          "--diff-cmd diff --extensions '-bBwEu'" : "";
         if (file2)
-          snprintf(command, sizeof(command), "svn diff -r \"%s\" \"%s\"",
-	           file2 + 1, file1);
+          snprintf(command, sizeof(command), "svn diff %s -r \"%s\" \"%s\"", 
+	           diffopt, file2 + 1, file1);
 	else
-          snprintf(command, sizeof(command), "svn diff \"%s\"", file1);
+          snprintf(command, sizeof(command), "svn diff %s \"%s\"",
+	           diffopt, file1);
       }
       else
         return (false);
     }
   }
 
-  diff = popen(command, "r");
-  if (!diff)
+  if (!pdiff.popen(command))
   {
     fl_alert("Diff command failed!\n\n%s", command);
     return (false);
@@ -1002,24 +1027,24 @@ DiffView::load(const char *file1,	// I - First file
     if (directories_)
     {
       snprintf(command, sizeof(command), "ls -a1 \"%s\"", file2);
-      fp = popen(command, "r");
+      pfp.popen(command, "r");
     }
     else
-      fp = fopen(file2, "r");
+      pfp.fopen(file2, "r");
   }
   else if (directories_)
   {
     snprintf(command, sizeof(command), "ls -a1 \"%s\"", file1);
-    fp = popen(command, "r");
+    pfp.popen(command, "r");
   }
   else
-    fp = fopen(file1, "r");
+    pfp.fopen(file1, "r");
 
-  if (!fp)
+  if (!pfp.is_open())
   {
     fl_alert("Unable to open \"%s\":\n\n%s", file2 ? file2 : file1,
              strerror(errno));
-    pclose(diff);
+    pdiff.close();
     return (false);
   }
 
@@ -1028,7 +1053,7 @@ DiffView::load(const char *file1,	// I - First file
 
   if (directories_)
   {
-    while (fgets(line, sizeof(line), fp))
+    while (pfp.get_line(line, sizeof(line)))
     {
       ptr = line + strlen(line) - 1;
       if (ptr >= line && *ptr == '\n')
@@ -1038,7 +1063,7 @@ DiffView::load(const char *file1,	// I - First file
     }
   }
 
-  while (fgets(line, sizeof(line), diff))
+  while (pdiff.get_line(line, sizeof(line)))
   {
     if (directories_)
     {
@@ -1222,7 +1247,7 @@ DiffView::load(const char *file1,	// I - First file
 
       while (fplinenum < newstart)
       {
-	if (!fgets(line, sizeof(line), fp))
+	if (!pfp.get_line(line, sizeof(line)))
           break;
 
 	add_line(line, line, false);
@@ -1233,7 +1258,7 @@ DiffView::load(const char *file1,	// I - First file
 
       while (count > 0 || newcount > 0)
       {
-	if (!fgets(line, sizeof(line), diff))
+	if (!pdiff.get_line(line, sizeof(line)))
           break;
 
 	if (line[0] == '+')
@@ -1262,23 +1287,23 @@ DiffView::load(const char *file1,	// I - First file
 
 	if (line[0] != '-')
 	{
-          fgets(line, sizeof(line), fp);
+          pfp.get_line(line, sizeof(line));
           fplinenum ++;
 	}
       }
     }
   }
 
-  pclose(diff);
+  pdiff.close();
 
   if (directories_)
-    pclose(fp);
+    pfp.close();
   else
   {
-    while (fgets(line, sizeof(line), fp) != NULL)
+    while (pfp.get_line(line, sizeof(line)) != NULL)
       add_line(line, line, false);
 
-    fclose(fp);
+    pfp.close();
   }
 
   fl_font(textfont_, textsize_);
@@ -1517,5 +1542,5 @@ DiffView::timeout_cb(DiffView *d)	// I - DiffView widget
 
 
 //
-// End of "$Id$".
+// End of "$Id: DiffView.cxx 407 2006-11-13 18:54:02Z mike $".
 //
