@@ -1,9 +1,9 @@
 //
-// "$Id$"
+// "$Id: DiffWindow.cxx 407 2006-11-13 18:54:02Z mike $"
 //
 // DiffWindow widget code.
 //
-// Copyright 2005 by Michael Sweet.
+// Copyright 2005-2006 by Michael Sweet.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License v2 as published
@@ -50,14 +50,21 @@
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Help_View.H>
-#include <FL/Fl_Int_Input.H>
+#include <FL/Fl_Spinner.H>
 #include <FL/fl_ask.H>
 #include <FL/fl_show_colormap.H>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+
+#if defined(WIN32)
+#include <io.h>
+#define snprintf _snprintf
+#define access _access
+#else
 #include <unistd.h>
+#endif
 
 
 //
@@ -133,31 +140,32 @@ DiffWindow::DiffWindow(const char *file1,
 
   resizable(&view_);
 
-  menubar_.add("&File/Open\\/Compare...", FL_CTRL | 'o',
+  menubar_.box(FL_THIN_UP_BOX);
+  menubar_.add("&File/Open\\/Compare...", FL_COMMAND | 'o',
                (Fl_Callback *)compare_cb, this);
-  menubar_.add("&File/Open\\/Compare &Selected", FL_CTRL | 'y',
+  menubar_.add("&File/Open\\/Compare &Selected", FL_COMMAND | 'y',
                (Fl_Callback *)compare_selected_cb, this);
-  menubar_.add("&File/&Close", FL_CTRL | 'w',
+  menubar_.add("&File/&Close", FL_COMMAND | 'w',
                (Fl_Callback *)close_cb, this, FL_MENU_DIVIDER);
-  menubar_.add("&File/&Redo Diff", FL_CTRL | 'r',
+  menubar_.add("&File/&Redo Diff", FL_COMMAND | 'r',
                (Fl_Callback *)rediff_cb, this, FL_MENU_DIVIDER);
-  menubar_.add("&File/&Quit", FL_CTRL | 'q',
+  menubar_.add("&File/&Quit", FL_COMMAND | 'q',
                (Fl_Callback *)quit_cb, this);
 
-  menubar_.add("&Edit/Copy", FL_CTRL | 'c',
+  menubar_.add("&Edit/Copy", FL_COMMAND | 'c',
                (Fl_Callback *)copy_cb, this, FL_MENU_DIVIDER);
-  menubar_.add("&Edit/Select Left", FL_CTRL | 'a',
+  menubar_.add("&Edit/Select Left", FL_COMMAND | 'a',
                (Fl_Callback *)select_left_cb, this);
-  menubar_.add("&Edit/Select Right", FL_CTRL | 'A',
+  menubar_.add("&Edit/Select Right", FL_COMMAND | 'A',
                (Fl_Callback *)select_right_cb, this);
-  menubar_.add("&Edit/Select None", FL_CTRL | 'n',
+  menubar_.add("&Edit/Select None", FL_COMMAND | 'n',
                (Fl_Callback *)select_none_cb, this, FL_MENU_DIVIDER);
-  menubar_.add("&Edit/Preferences", 0,
+  menubar_.add("&Edit/Preferences...", 0,
                (Fl_Callback *)prefs_cb, this);
 
-  menubar_.add("&Search/Find...", FL_CTRL | 'f',
+  menubar_.add("&Search/Find...", FL_COMMAND | 'f',
                (Fl_Callback *)find_cb, this);
-  menubar_.add("&Search/Find Next", FL_CTRL | 'g',
+  menubar_.add("&Search/Find Next", FL_COMMAND | 'g',
                (Fl_Callback *)find_next_cb, this, FL_MENU_DIVIDER);
   menubar_.add("&Search/Previous Change", 'p',
                (Fl_Callback *)prev_change_cb, this);
@@ -243,8 +251,8 @@ void
 DiffWindow::about_cb(Fl_Menu_Bar *m,	// I - Menubar
                      DiffWindow  *dw)	// I - Window
 {
-  fl_message(VERSION "\nCopyright 2005 by Michael Sweet\n"
-             "This program is free software provided under the\n"
+  fl_message(VERSION "\nCopyright 2005-2006 by Michael Sweet\n"
+	     "This program is free software provided under the\n"
 	     "terms of the GNU General Public License v2.");
 }
 
@@ -304,7 +312,7 @@ DiffWindow::compare_cb(Fl_Menu_Bar *m,	// I - Menubar
   c1 = dow->dc1()->count();
   c2 = dow->dc2()->count();
 
-  if (c2 <= 1)
+  if (c1 > 0 && c2 <= 1)
   {
     // Compare N files against a single file/directory.
     f2 = dow->dc2()->value();
@@ -334,7 +342,7 @@ DiffWindow::compare_cb(Fl_Menu_Bar *m,	// I - Menubar
 	}
     }
   }
-  else
+  else if (c1 && c2)
   {
     // Compare against multiple files/directories...
     if (c1 <= 1)
@@ -854,7 +862,7 @@ DiffWindow::load_prefs()
   selection_color((Fl_Color)v);
 
   prefs_.get("showlinenum", v, true);
-  showlinenum(v);
+  showlinenum(v ? true : false);
 
   prefs_.get("tabwidth", v, 8);
   tabwidth(v);
@@ -864,6 +872,9 @@ DiffWindow::load_prefs()
 
   prefs_.get("textsize", v, FL_NORMAL_SIZE);
   textsize(v);
+
+  prefs_.get("ignoreblanks", v, false);
+  ignoreblanks(v ? true : false);
 }
 
 
@@ -940,54 +951,69 @@ DiffWindow::prefs_cb(Fl_Menu_Bar *m,	// I - Menubar
                      DiffWindow  *dw)	// I - Window
 {
   Fl_Double_Window	*win;		// Preferences window
-  Fl_Int_Input		*tabwidth,	// Tab size
+  Fl_Group		*group;		// Control group
+  Fl_Spinner		*tabwidth,	// Tab size
 			*textsize;	// Text size
   Fl_Button		*textfg,	// Foreground color
 			*textbg,	// Background color
 			*hicolor;	// Highlight color
   Fl_Check_Button	*linenum;	// Show line numbers?
+  Fl_Check_Button	*ignoreb;	// ignore blanks during diff compare ?
   Fl_Button		*ok,		// OK button
 			*cancel;	// Cancel button
   Fl_Widget		*current;	// Current widget
-  char			s[255];		// Temporary string
 
 
-  win = new Fl_Double_Window(200, 265, "Preferences - " VERSION);
+  win = new Fl_Double_Window(220, 310, "Preferences - " VERSION);
   win->modal();
 
-  tabwidth = new Fl_Int_Input(150, 10, 40, 25, "Tab Width:");
-  tabwidth->align(FL_ALIGN_LEFT);
-  sprintf(s, "%d", dw->view_.tabwidth());
-  tabwidth->value(s);
+    group = new Fl_Group(10, 10, 200, 255);
+    group->box(FL_THIN_DOWN_BOX);
+    group->color(FL_GRAY - 1);
 
-  textsize = new Fl_Int_Input(150, 45, 40, 25, "Text Size:");
-  textsize->align(FL_ALIGN_LEFT);
-  sprintf(s, "%d", dw->view_.textsize());
-  textsize->value(s);
+      tabwidth = new Fl_Spinner(160, 20, 40, 25, "Tab Width:");
+      tabwidth->align(FL_ALIGN_LEFT);
+      tabwidth->range(1.0, 20.0);
+      tabwidth->step(1.0);
+      tabwidth->value(dw->view_.tabwidth());
 
-  textfg = new Fl_Button(150, 80, 25, 25, "Text Color:");
-  textfg->align(FL_ALIGN_LEFT);
-  textfg->color(dw->view_.textcolor());
-  textfg->box(FL_BORDER_BOX);
+      textsize = new Fl_Spinner(160, 55, 40, 25, "Text Size:");
+      textsize->align(FL_ALIGN_LEFT);
+      textsize->range(6.0, 20.0);
+      textsize->step(1.0);
+      textsize->value(dw->view_.textsize());
 
-  textbg = new Fl_Button(150, 115, 25, 25, "Background Color:");
-  textbg->align(FL_ALIGN_LEFT);
-  textbg->color(dw->view_.color());
-  textbg->box(FL_BORDER_BOX);
+      textfg = new Fl_Button(160, 90, 25, 25, "Text Color:");
+      textfg->align(FL_ALIGN_LEFT);
+      textfg->color(dw->view_.textcolor());
+      textfg->box(FL_BORDER_BOX);
 
-  hicolor = new Fl_Button(150, 150, 25, 25, "Highlight Color:");
-  hicolor->align(FL_ALIGN_LEFT);
-  hicolor->color(dw->view_.selection_color());
-  hicolor->box(FL_BORDER_BOX);
+      textbg = new Fl_Button(160, 125, 25, 25, "Background Color:");
+      textbg->align(FL_ALIGN_LEFT);
+      textbg->color(dw->view_.color());
+      textbg->box(FL_BORDER_BOX);
 
-  linenum = new Fl_Check_Button(150, 185, 25, 25, "Show Line Numbers:");
-  linenum->align(FL_ALIGN_LEFT);
-  linenum->value(dw->view_.showlinenum());
+      hicolor = new Fl_Button(160, 160, 25, 25, "Highlight Color:");
+      hicolor->align(FL_ALIGN_LEFT);
+      hicolor->color(dw->view_.selection_color());
+      hicolor->box(FL_BORDER_BOX);
 
-  ok = new Fl_Button(80, 230, 40, 25, "OK");
-  cancel = new Fl_Button(130, 230, 60, 25, "Cancel");
+      linenum = new Fl_Check_Button(160, 195, 25, 25, "Show Line Numbers:");
+      linenum->align(FL_ALIGN_LEFT);
+      linenum->value(dw->view_.showlinenum());
+
+      ignoreb = new Fl_Check_Button(160, 230, 25, 25, "Ignore Whitespace:");
+      ignoreb->align(FL_ALIGN_LEFT);
+      ignoreb->value(dw->view_.ignoreblanks());
+
+    group->end();
+
+    ok = new Fl_Button(100, 275, 40, 25, "OK");
+    ok->shortcut(FL_Enter);
+    cancel = new Fl_Button(150, 275, 60, 25, "Cancel");
 
   win->end();
+  win->hotspot(win);
   win->show();
 
   while (win->shown())
@@ -999,10 +1025,11 @@ DiffWindow::prefs_cb(Fl_Menu_Bar *m,	// I - Menubar
       // Apply prefs
       dw->view_.color(textbg->color());
       dw->view_.textcolor(textfg->color());
-      dw->view_.tabwidth(atoi(tabwidth->value()));
-      dw->view_.textsize(atoi(textsize->value()));
+      dw->view_.tabwidth((int)tabwidth->value());
+      dw->view_.textsize((int)textsize->value());
       dw->view_.selection_color(hicolor->color());
-      dw->view_.showlinenum(linenum->value());
+      dw->view_.showlinenum(linenum->value() ? true : false);
+      dw->view_.ignoreblanks(ignoreb->value() ? true : false);
 
       // Reload the diff...
       dw->view_.load(dw->file1_, dw->file2_);
@@ -1105,6 +1132,7 @@ DiffWindow::save_prefs()
   prefs_.set("tabwidth", tabwidth());
   prefs_.set("textcolor", textcolor());
   prefs_.set("textsize", textsize());
+  prefs_.set("ignoreblanks", ignoreblanks());
 }
 
 
@@ -1273,5 +1301,5 @@ DiffWindow::using_cb(Fl_Menu_Bar *m,	// I - Menubar
 
 
 //
-// End of "$Id$".
+// End of "$Id: DiffWindow.cxx 407 2006-11-13 18:54:02Z mike $".
 //
