@@ -1,5 +1,5 @@
 //
-// "$Id$"
+// "$Id: fldiff.cxx 407 2006-11-13 18:54:02Z mike $"
 //
 // A graphical diff program.
 //
@@ -39,8 +39,8 @@ main(int  argc,				// I - Number of command-line args
   // Use POSIX locale for all commands...
   putenv("LANG=C");
 
-  // Use the plastic scheme...
-  Fl::scheme("plastic");
+  // Use the GTK+ scheme...
+  Fl::scheme("gtk+");
 
   // Create the window...
   if (argc > 1)
@@ -59,5 +59,5 @@ main(int  argc,				// I - Number of command-line args
 
 
 //
-// End of "$Id$".
+// End of "$Id: fldiff.cxx 407 2006-11-13 18:54:02Z mike $".
 //
